﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;
using Windows.Devices.Sensors;
using Windows.UI.Xaml.Media.Animation;


namespace CompassApp
{
    /// <summary>
    /// 可用于自身或导航至 Frame 内部的空白页。
    /// </summary>
    public sealed partial class MainPage : Page
    {
        RotateTransform rotatTransform = null;

        uc1 uc = null;
        Flyout fly = null;

        bool isChecking = false;

        public MainPage ()
        {
            this.InitializeComponent();
            _compass = Compass.GetDefault(); //获取罗盘传感器控制对象实例
            uint v = _compass.MinimumReportInterval;
            uint Interval = 20;
            // 设置报告数值的时间间隔，以毫秒为单位
            // 设置的间隔不能小于MinimumReportInterval的值
            _compass.ReportInterval = (Interval <= v) ? v : Interval;

            rotatTransform = new RotateTransform();
            ge.RenderTransform = rotatTransform;

            Window.Current.VisibilityChanged += OnWindowVisibilityChanged;

            this.NavigationCacheMode = NavigationCacheMode.Required;

            fly = new Flyout();
            uc = new uc1();
            uc.Height = 300;
            fly.Content = uc;
            fly.Opened += fly_Opened;
            fly.Closed += fly_Closed;
        }

        void fly_Closed ( object sender, object e )
        {
            Windows.Phone.UI.Input.HardwareButtons.BackPressed -= OnBackButton;
        }

        void fly_Opened ( object sender, object e )
        {
            Windows.Phone.UI.Input.HardwareButtons.BackPressed += OnBackButton;
        }

        void OnBackButton ( object sender, Windows.Phone.UI.Input.BackPressedEventArgs e )
        {
            fly.Hide();
            isChecking = false;
            e.Handled = true;
        }

        void OnWindowVisibilityChanged ( object sender, Windows.UI.Core.VisibilityChangedEventArgs e )
        {
            if (e.Visible)
            {
                var acc = _compass.GetCurrentReading().HeadingAccuracy;
                if (acc != MagnetometerAccuracy.High)
                {
                    // 校准
                    fly.ShowAt(this);
                    isChecking = true;
                }
                _compass.ReadingChanged += _compass_ReadingChanged;
            }
            else
            {
                _compass.ReadingChanged -= _compass_ReadingChanged;
            }
        }

        async void _compass_ReadingChanged ( Compass sender, CompassReadingChangedEventArgs args )
        {
            var res = args.Reading;
            // 如果地北极的偏角值不可用，则使用磁北极角度
            double val = res.HeadingTrueNorth ?? res.HeadingMagneticNorth;
            await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal,
                () =>
                {
                    // 如果精度较高，则停止校准
                    if (isChecking && res.HeadingAccuracy == MagnetometerAccuracy.High)
                    {
                        fly.Hide();
                        isChecking = false;
                    }

                    // 旋转的方向是罗盘角度的负值
                    rotatTransform.Angle = -val;
                    Display(val);
                });
        }

        Compass _compass = null;

        /// <summary>
        /// 显示方位
        /// </summary>
        private void Display ( double v )
        {
            string d = "";
            int ind = Convert.ToInt32(v);
            if (ind == 0 || ind == 360)
            {
                d = "正北";
            }
            else if (ind == 90)
            {
                d = "正东";
            }
            else if (ind == 180)
            {
                d = "正南";
            }
            else if (ind == 270)
            {
                d = "正西";
            }
            else if (ind > 0 && ind < 90)
            {
                d = "东北";
            }
            else if (ind > 90 && ind < 180)
            {
                d = "东南";
            }
            else if (ind > 180 && ind < 270)
            {
                d = "西南";
            }
            else if (ind > 270 && ind < 360)
            {
                d = "西北";
            }
            tbW.Text = string.Format("{0}°（{1}）", ind, d);
        }

    }
}
